#include <qfile.h>
#include <qsplitter.h>
#include <qtabwidget.h>
#include <qtextview.h>

#include <kaction.h>
#include <kapplication.h>
#include <kdebug.h>
#include <kfiledialog.h>
#include <kmimetype.h>
#include <klistview.h>
#include <klocale.h>
#include <kstatusbar.h>
#include <kstdaction.h>

#include "utils/domtreepart.h"
#include "utils/jsconsolepart.h"
#include "utils/dialogfactory.h"
#include "utils/messagelogwidget.h"

#include "doctype.h"
#include "doctypemanager.h"
#include "doctypeviewmanager.h"
#include "document.h"

#include "xmelegance.h"
#include "xmelegance.moc"

XMElegance::XMElegance()
    : KParts::MainWindow( 0, "XMElegance" )
{
    // Setup managers
    partManager = new KParts::PartManager( this );
    docTypeMng = new DocTypeManager( this );

    // Setup Views
    sidebarSplit = new QSplitter( this );
    setupSidebar( sidebarSplit );

    messageSplit = new QSplitter( Qt::Vertical, sidebarSplit );
    setupDocViews( messageSplit );
    setupMessageViews( messageSplit );

    // Setup main window
    setCentralWidget( sidebarSplit );
    statusBar()->message( "XMElegance", 2000 );
    setupActions();

    installBindings();

    // Activate KParts
    createGUI( 0 );
    connect( partManager, SIGNAL( activePartChanged( KParts::Part * ) ),
	     this, SLOT( createGUI( KParts::Part * ) ) );
}

XMElegance::~XMElegance()
{
}

void XMElegance::activeViewChanged( QWidget *w )
{
    KParts::Part *p = (docViewMng ? docViewMng->findPart( w ) : 0);
    if ( p ) {
	partManager->setActivePart( p );
	return;
    }

    if ( w == jsConsole->widget() ) {
	partManager->setActivePart( jsConsole );
	return;
    }

    if ( w == domTree->widget() ) {
	partManager->setActivePart( domTree );
	return;
    }

    partManager->setActivePart( 0 );
}

void XMElegance::setupActions()
{
    // Actions
    KStdAction::open( this, SLOT( open() ), actionCollection() );
    KStdAction::quit( this, SLOT( close() ), actionCollection() );

    recent_files = KStdAction::openRecent( this, SLOT( openURL(const KURL &) ),
					   actionCollection() );
    recent_files->loadEntries( kapp->config() );

    //XXX: Remove manual icon loading
    QPixmap sbpm( "tree_win.png" );
    show_sidebar = new KToggleAction( i18n( "Show &Sidebar" ), QIconSet(sbpm), CTRL+Key_T,
				      0, 0,
				      actionCollection(), "show_hide_sidebar" );
    show_sidebar->setChecked( true );
    show_sidebar->setStatusText( i18n("Shows or hides the sidebar") );
    connect( show_sidebar, SIGNAL( toggled(bool) ),
	     this, SLOT( setSidebarVisible(bool) ) );

    QPixmap smpm( "output_win.png" );
    show_messages = new KToggleAction( i18n( "Show &Messages" ), QIconSet(smpm), CTRL+Key_M,
				       0, 0,
				       actionCollection(), "show_hide_messages" );
    show_messages->setChecked( true );
    show_messages->setStatusText( i18n("Shows or hides the message pane") );
    connect( show_messages, SIGNAL( toggled(bool) ),
	     this, SLOT( setMessagesVisible(bool) ) );

    actionCollection()->setHighlightingEnabled( true );
    connect( actionCollection(), SIGNAL( actionStatusText(const QString &) ),
	     statusBar(), SLOT( message(const QString &) ) );
    connect( actionCollection(), SIGNAL( clearStatusText() ),
	     statusBar(), SLOT( clear() ) );
}

void XMElegance::installBindings()
{
    KJScript *js = jsConsole->jscript();
    KJS::Imp *imp = js->globalObject();
    imp->put( "execDialog", KJS::Function( new JSDialogFactoryExec() ) );
    imp->put( "showDialog", KJS::Function( new JSDialogFactoryShow() ) );
}

void XMElegance::openURL( const KURL &url )
{
    // DocType
    KMimeType::Ptr mimeType = KMimeType::findByURL( url );
    if (!mimeType) {
	kdDebug() << "openURL for " << url.url() << " could not find out mimetype" << endl;
	return;
    }
    kdDebug() << "openURL for " << url.url() << " found mimetype " << mimeType->name() << endl;

    // TODO: Remove this hack!
    if ( url.url().endsWith("docbook") )
	installDocType( "text/docbook-xml" );
    else
	installDocType( mimeType->name() );

    if ( docViewMng ) {
	recent_files->addURL( url );
	recent_files->saveEntries( kapp->config() );

	docViewMng->openURL( url );
	domTree->openURL( url );
    }
}


void XMElegance::open()
{
    KURL url = KFileDialog::getOpenURL( ":document" );
    if ( !url.isEmpty() )
	openURL( url );
}

void XMElegance::setSidebarVisible( bool visible )
{
    if ( visible && sidebar->isHidden() )
	sidebar->show();
    else if ( sidebar->isVisible() )
	sidebar->hide();
}

void XMElegance::setMessagesVisible( bool visible )
{
    if ( visible && messages->isHidden() )
	messages->show();
    else if ( messages->isVisible() )
	messages->hide();
}

//
// Sidebar Views
//

void XMElegance::setupSidebar( QWidget *parent )
{
    // Sidebar container
    sidebar = new QTabWidget( parent, "sidebar" );
    sidebar->setMargin( 0 );
    sidebar->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding,
					 QSizePolicy::MinimumExpanding ) );
    connect( sidebar, SIGNAL( currentChanged( QWidget * ) ),
	     this, SLOT( activeViewChanged( QWidget * ) ) );

    // Sidebars
    domTree = new DOMTreePart( sidebar, "dom_tree" );
    sidebar->addTab( domTree->widget(), "&Doc Tree" );
    partManager->addPart( domTree );

    QTextView *test = new QTextView( sidebar );
    test->setText( "This is some text...\n"
		   "This is some text...\n"
		   "This is some text...\n"
		   "This is some text...\n"
		   "This is some text...\n"
		   "This is some text...\n" );
    sidebar->addTab( test, "Test" );
}

//
// Document Views
//

void XMElegance::setupDocViews( QWidget *parent )
{
    // View container
    docViews = new QTabWidget( parent, "docView" );
    docViews->setMargin( 0 );
    docViews->setSizePolicy( QSizePolicy( QSizePolicy::Expanding,
					  QSizePolicy::Expanding ) );
    connect( docViews, SIGNAL( currentChanged( QWidget * ) ),
	     this, SLOT( activeViewChanged( QWidget * ) ) );
}

void XMElegance::installDocType( const QString &mime )
{
    kdDebug() << "installDocType() " << mime << endl;

    if ( docViewMng ) {
	docViewMng->uninstallViews( docViews, partManager );
	delete docViewMng;
    }

    kdDebug() << "docViewMng about to activate " << mime << endl;

    //// Install the DocType for the specified mime type
    if ( !docTypeMng->activate(mime) )
	return;

    kdDebug() << "docViewMng activated ok " << mime << endl;

    docViewMng = docType()->createViewManager();

    kdDebug() << "docViewMng created view manager " << mime << endl;

    docViewMng->installViews( docViews, partManager );
    docViews->updateGeometry();
    activeViewChanged( docViews->currentPage() );

    QString msg( i18n("%1 (%2)") );
    QString s = msg.arg( docType()->info().description() ).arg( docType()->info().mimeType() );
    statusBar()->message( s );
}

//
// Message Views
//

void XMElegance::setupMessageViews( QWidget *parent )
{
    // Message view container
    messages = new QTabWidget( parent, "messages" );
    messages->setMargin( 0 );
    messages->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding,
					  QSizePolicy::MinimumExpanding ) );
    connect( messages, SIGNAL( currentChanged( QWidget * ) ),
	     this, SLOT( activeViewChanged( QWidget * ) ) );

    // Messages view
    msgView = new MessageLogWidget( messages );
    msgView->message( i18n("<b>XMElegance started</b><br>") );
    messages->addTab( msgView, "&Messages" );

    // JSConsole view
    jsConsole = new JSConsolePart( messages );
    messages->addTab( jsConsole->widget(), "&JScript" );
    partManager->addPart( jsConsole );
}

DocType *XMElegance::docType() const
{
    return docTypeMng->docType();
}

// Local Variables:
// c-basic-offset: 4
// End:

