#include <assert.h>

#include <kconfig.h>
#include <kdebug.h>

#include "doctypeinfo.h"
#include "doctype.h"
#include "global.h"
#include "xmelegance.h"

#include "doctypes/docbookdoctype.h"
#include "doctypes/htmldoctype.h"
#include "doctypes/qmldoctype.h"
#include "doctypes/rdfdoctype.h"
#include "doctypes/xmldoctype.h"

#include "doctypemanager.h"
#include "doctypemanager.moc"

DocTypeManager::DocTypeManager( QObject *obj, const char *name )
  : QObject( obj, name ? name : "doctype_manager" ),
    infoMap(), loadedMap()
{
    kdDebug() << "DocTypeManager created" << endl;
    loadDocTypeInfo();
}

DocTypeManager::~DocTypeManager()
{
    QMapIterator<DocTypeInfo, DocType *> it;
    for ( it = loadedMap.begin(); it != loadedMap.end(); ++it ) {
	delete it.data();
    }
}

void DocTypeManager::loadDocTypeInfo()
{
    add( DocTypeInfo("docbook", "DocBook", "DocBook XML", "text/docbook-xml") );
    add( DocTypeInfo("html", "HTML", "Hypertext Markup Language", "text/html") );
    add( DocTypeInfo("qml", "QML", "Qt Internal Markup", "text/x-qml") );
    add( DocTypeInfo("rdf", "RDF", "Resource Description Framework", "text/rdf") );
    add( DocTypeInfo("xml", "XML", "Generic XML DocType", "text/xml") );
}

void DocTypeManager::add( const DocTypeInfo &info )
{
    infoMap[ info.mimeType() ] = info;
}

const DocTypeInfo &DocTypeManager::infoForMimeType( const QString &mime )
{
    kdDebug() << "DocTypeManager::infoForMimeType() called for " << mime << endl;

    if ( infoMap.count( mime ) )
	return infoMap[ mime ];
    return infoMap[ "text/xml" ];
}

DocType *DocTypeManager::docType( const QString &mime )
{
    return docType( infoForMimeType( mime ) );
}

DocType *DocTypeManager::docType( const DocTypeInfo &info )
{
    kdDebug() << "DocTypeManager::docType() called for info " << info.mimeType() << endl;

    if ( loadDocType( info ) ) {
	// We have now loaded this doctype
	DocType *dt = loadedMap[ info ];

	if (dt)
	    kdDebug() << "DocTypeManager::docType() succeeded" << endl;
	else
	    kdDebug() << "DocTypeManager::docType() failed" << endl;

	return dt;
    }

    return 0;
}

bool DocTypeManager::loadDocType( const DocTypeInfo &info )
{
    DocType *dt = loadedMap[ info ];
    if ( dt )
	return true;

    // Create DocType object
    QString mime = info.mimeType();
    DocType *currType;
    if ( mime == "text/html" )
  	currType = new HTMLDocType( info );
    else if ( mime == "text/x-qml" )
  	currType = new QMLDocType( info );
    else if ( mime == "text/rdf" )
  	currType = new RDFDocType( info );
    else if ( mime == "text/docbook-xml" )
  	currType = new DocBookDocType( info );
    else
  	currType = new XMLDocType( info );

    loadedMap[ info ] = currType;
    currType->onLoad();

    return true;
}

bool DocTypeManager::unloadDocType( const DocTypeInfo &info )
{
    DocType *dt = loadedMap[ info ];
    if ( !dt )
	return false;

    dt->onUnload();
    Global::xmel()->factory()->removeClient( dt );
    loadedMap.remove( info );
    delete dt;

    return true;
}

// Local Variables:
// c-basic-offset: 4
// End:

